PRO PKWINDS,kinc,sflg,offset,flcnt
; This module produces an output file
; which lists the peak marine and open
; terrain wind components experienced at
; each zipcode for the current storm. If
; the storm makes landfall, then the peak
; marine and open terrain winds are listed
; at the time and site of landfall. If the
; storm only bypasses the state then the
; peak marine and open terrain winds are 
; listed for the fix exhibiting the lowest
; central pressure

compile_opt hidden
on_error,1

;...preamble
if n_params() eq 0 then begin
print,'PKWINDS,kinc,sflg,offset,flcnt'
return & endif

;...annunciation
print,'COMMENCING PEAK WINDS LISTING'

;...initialization
restore,'zipcodes.idl' & restore,'fixshots.idl'
kmax=n_elements(elonk) & zuvzip=complexarr(nzip,kmax/kinc+1)
werzipx=fltarr(nzip) & szip=werzipx & zuvzipx=complexarr(nzip)
unow=fltarr(pixr,pixa) & vnow=unow

;...calculate all time series
jj=-1 & for k=0,kmax-1,kinc do begin jj=jj+1
elc=elonk(k) & nlc=nlatk(k) & rmw=rmwk(k)

;...determine which zipcodes will be affected by current storm position
for i=0,nzip-1 do $
szip(i)=MAP_2POINTS(elc,nlc,elzip(i),nlzip(i),/meters)/(1e3*rmw)
reachx=REACH(rmw) & reachx=reachx(0) & reachx=min([reachx,12.5])
izip=where(szip le reachx,ict)
if ict eq 0  then goto,labela

;...generate relavent portion of gridded field for current time (k)
for i=0,pixr-1 do for j=0,pixa-1 do begin
kz=k
unow(i,j)=spl_interp(ktime,usnap(i,j,*),d2usnap(i,j,*),kz)
vnow(i,j)=spl_interp(ktime,vsnap(i,j,*),d2vsnap(i,j,*),kz)
endfor

;...evaluate marine windfield components at admissible zipcode centroids
elz=elzip(izip) & nlz=nlzip(izip)
LLTOXY,elz,nlz,xmerc,ymerc,elc,nlc
sdx=sqrt(xmerc^2+ymerc^2)/(100*rmw)
odx=18*(atan(ymerc,xmerc)/!pi+1)
;print,min(sdx),max(sdx),min(odx),max(odx)
zuvnow=complex(unow(0:pixr-1,1:pixa-2),vnow(0:pixr-1,1:pixa-2))
zerzip=interpolate(zuvnow,sdx,odx,cubic=-.5)

;...archive the "open terrain" wind components in array format
ZMAR2ZOT,zerzip,zotzip
zuvzip(izip,jj)=zotzip & labela: & endfor
wzip=float(sqrt(zuvzip*conj(zuvzip)))

;...record maximum total "ot" windspeed at each zipcode
for i=0,nzip-1 do werzipx(i)=max(wzip(i,*)) & index=where(werzipx ne 0.,ict) & sflg=0 
for j=0,ict-1 do begin  jct=where(wzip(index(j),*) eq werzipx(index(j)))
zuvzipx(index(j))=zuvzip(index(j),jct) &endfor

;...obtain marine and "open terrain" peak winds at landfall or lowest pressure fix
ilf=where(lsflg eq 1,jct) & if jct eq 0 then ilf=where(lsflg eq 4)

for jj=1,n_elements(ilf) do begin
zsnaplf=zsnap(*,*,ilf(jj-1))
wsnaplf=float(sqrt(zsnaplf*conj(zsnaplf)))

;...marine winds to open terrain winds
ZMAR2ZOT,zsnaplf,zotlf

;...peak winds at landfall or lowest pressure fix
wotlf=float(sqrt(zotlf*conj(zotlf)))
wslfx=max(wsnaplf) & iwslfx=where(wsnaplf eq wslfx)
wotlfx=max(wotlf)  & iwotlfx=where(wotlf eq wotlfx)
zsnaplfx=zsnaplf(iwslfx) & zotlfx=zotlf(iwotlfx)

;...time and location of landfall or lowest pressure fix
daylf=day(ilf(jj-1)) & hourlf=zhour(ilf(jj-1)) & minlf=zmin(ilf(jj-1))
elonlf=elon(ilf(jj-1)) & nlatlf=nlat(ilf(jj-1))

;...make file if any zipcodes are affected by this cyclone
;if ict eq 0 then goto,labelb 
if jj eq 1 then begin
flcnt=flcnt+1 & sflg=ict
offset=fix(60*zhour(0)+zmin(0)) &endif

;...write and label current output file
lbl='landfall:' & if jct eq 0 then lbl='bypass:'
if jj eq 1 then openw,lu,'output'+strcompress(flcnt,/REMOVE_ALL)+'.dat',/get_lun
header='ter      day    hour     min      zonal     meridional    total m/s dir(deg)'
printf,lu,string
printf,lu,lbl+'  longitude:',elonlf,' deg   latitude:',nlatlf,' deg'
printf,lu,header
printf,lu,' MA',fix(daylf),fix(hourlf),fix(minlf),float(zsnaplfx(0)),imaginary(zsnaplfx(0)),wslfx(0),$
fix(180*(atan(float(zsnaplfx(0)),imaginary(zsnaplfx(0)))/!pi+1.))
printf,lu,' OT',fix(daylf),fix(hourlf),fix(minlf),float(zotlfx(0)),imaginary(zotlfx(0)),wotlfx(0),$
fix(180*(atan(float(zotlfx(0)),imaginary(zotlfx(0)))/!pi+1.))
endfor

for i=0,sflg-1 do begin ixi=where(wzip(index(i),*) ne 0.) 
jxj=where(wzip(index(i),ixi) eq werzipx(index(i)))  

printf,lu,' '
printf,lu,'zipcode:',zip(index(i)),'  longitude:',elzip(index(i)),$
' deg   latitude:',nlzip(index(i)),' deg'

dayz=1+(offset+kinc*ixi(jxj(0)))/1440. & hourz=24*(dayz-fix(dayz))
minz=60*(hourz-fix(hourz)) & jxm=where(fix(round(minz)) eq 60,jct)
if jct ne 0 then begin minz(jxm)=0. & hourz(jxm)=1.+hourz(jxm) &endif
printf,lu,header
printf,lu,' OT',fix(dayz),fix(hourz),fix(round(minz)),$  
float(zuvzip(index(i),ixi(jxj(0)))),imaginary(zuvzip(index(i),ixi(jxj(0)))),$
wzip(index(i),ixi(jxj(0))),fix(180*(atan(float(zuvzip(index(i),ixi(jxj(0)))),$
imaginary(zuvzip(index(i),ixi(jxj(0)))))/!pi+1.)) &endfor
close,lu & free_lun,lu
labelb: & print,'PEAK WINDS LISTING COMPLETED' 
END ;PKWINDS.PRO
